%%%  Data: 15 kwietnia 2019
%%%  Autor: Adam Meissner (ameis@man.poznan.pl)
%%%
%%%  Uwagi wstpne
%%%  ==============
%%%  Formuy logiki pierwszego rzdu zapisuje si w notacji
%%%  przedstawionej w poniszej tabeli; symbole F oraz G
%%%  oznaczaj dowolne formuy skadowe.
%%%  
%%%  Schemat |
%%%  formuy | Opis
%%%  --------+---------------------------------------------
%%%  ~ F     | negacja F
%%%  F & G   | koniunkcja F i G
%%%  F v G   | alternatywa F i G
%%%  F -> G  | implikowanie G przez F
%%%  F <-> G | rwnowano F i G
%%%  (A x) F | kwantyfikacja oglna (zmiennej x w F)
%%%  (E x) F | kwantyfikacja szczegowa (zmiennej x w F)
%%%  
%%%  Przykad 1
%%%  ===========
%%%  Teoria:
%%%  (1) (A x)(p(x) v (q(x) & r(x)))
%%%  (2) (A x)(~p(x) v (q(x) & r(x)))
%%%  Hipoteza:
%%%  (A x)(q(x) & r(x))
%%%  
%%%  Wynikiem przeksztacenia aksjomatw teorii oraz hipotezy 
%%%  do postaci kontrapozytywowej jest poniszy zbir klauzul
%%%  prologowych. Symbol s jest sta Skolema.

p(X) :- not_q(X).
p(X) :- r(X).

q(X) :- not_p(X).
q(X) :- p(X).

r(X) :- not_p(X).
r(X) :- p(X).

not_p(X) :- not_q(X).
not_p(X) :- not_r(X).

query :- q(s), r(s).

%%%  W celu przetestowania przykadu naley wczyta zawarto
%%%  plikw 'pttp.pl' oraz 'exmp02.pl' do bazy danych systemu
%%%  SWI-Prolog, a nastpnie zada zapytanie 'prove(query)':
%%%  
%%%  1 ?- [pttp].
%%%  true.
%%%  
%%%  2 ?- [exmp02].
%%%  true.
%%%  
%%%  3 ?- prove(query).
%%%  The query is proven with the depth limit: 3.
%%%  true.
%%%  
%%%  4 ?- 
%%%  
%%%  Odpowied 'true' oznacza, e hipoteza zostaa udowodniona.
%%%  UWAGA: pliki 'pttp.pl' oraz 'exmp02.pl' musz znajdowa
%%%  si w katalogu roboczym systemu SWI-Prolog.
