%%%  Data: 15 kwietnia 2019
%%%  Autor: Adam Meissner (ameis@man.poznan.pl)
%%%
%%%  Uwagi wstpne
%%%  ==============
%%%  Formuy logiki pierwszego rzdu zapisuje si w notacji
%%%  przedstawionej w poniszej tabeli; symbole F oraz G
%%%  oznaczaj dowolne formuy skadowe.
%%%  
%%%  Schemat |
%%%  formuy | Opis
%%%  --------+---------------------------------------------
%%%  ~ F     | negacja F
%%%  F & G   | koniunkcja F i G
%%%  F v G   | alternatywa F i G
%%%  F -> G  | implikowanie G przez F
%%%  F <-> G | rwnowano F i G
%%%  (A x) F | kwantyfikacja oglna (zmiennej x w F)
%%%  (E x) F | kwantyfikacja szczegowa (zmiennej x w F)
%%%  
%%%  Przykad 3
%%%  ===========
%%%  Teoria:
%%%  (1) (A x)(A s)(A t)e(x,s) & e(x,t) <-> e(x,p(s,t))
%%%  (2) (A s)(A t)((A x)e(x,s) -> e(x,t)) <-> z(s,t)
%%%  Hipoteza:
%%%  (A s)(A t)z(p(s,t),s) & z(p(s,t),t)
%%%  
%%%  Intuicyjnie, teoria definiuje funkcj p(s,t) przekroju
%%%  zbiorw s i t (1) oraz relacj z(s,t) zawierania si 
%%%  zboru s w zbiorze t (2). Pojciem pierwotnym jest relacja
%%%  e(x,s) przynalenoci elementu x do zbioru s. Hipoteza
%%%  postuluje, e przekrj dwch zbiorw zawiera si w kadym
%%%  ze skadnikw.
%%%  
%%%  Wynikiem przeksztacenia aksjomatw teorii oraz hipotezy 
%%%  do postaci kontrapozytywowej jest poniszy zbir klauzul
%%%  prologowych. Symbole a i b s staymi Skolema, a symbol
%%%  f/2 jest funkcj Skolema.

e(X,S) :- e(X,p(S,_)).
e(X,T) :- e(X,p(_,T)).
e(X,T) :- e(X,S), z(S,T).
e(X,p(S,T)) :- e(X,S), e(X,T).
e(f(S,T),S) :- not_z(S,T).

z(S,T) :- not_e(f(S,T),S).
z(S,T) :- e(f(S,T),T).

not_e(X,S) :- e(X,T), not_e(X,p(S,T)).
not_e(X,T) :- e(X,S), not_e(X,p(S,T)).
not_e(X,p(_,T)) :- not_e(X,T).
not_e(X,S) :- not_e(X,T), z(S,T).
not_e(X,p(S,_)) :- not_e(X,S).
not_e(f(S,T),T) :- not_z(S,T).

not_z(S,T) :- e(X,S), not_e(X,T).

query :- z(p(a,b),a), z(p(a,b),b).

%%%  W celu przetestowania przykadu naley wczyta zawarto
%%%  plikw 'pttp.pl' oraz 'exmp03.pl' do bazy danych systemu
%%%  SWI-Prolog, a nastpnie zada zapytanie 'prove(query)':
%%%  
%%%  1 ?- [pttp].
%%%  true.
%%%  
%%%  2 ?- [exmp03].
%%%  true.
%%%  
%%%  3 ?- prove(query).
%%%  The query is proven with the depth limit: 4.
%%%  true.
%%%  
%%%  4 ?- 
%%%  
%%%  Odpowied 'true' oznacza, e hipoteza zostaa udowodniona.
%%%  UWAGA: pliki 'pttp.pl' oraz 'exmp03.pl' musz znajdowa
%%%  si w katalogu roboczym systemu SWI-Prolog.
