prove(Query) :-
   gen_depth(Depth),
   prove(Query,[],Depth),
   format('The query is proven with the depth limit: ~w.~n', Depth),!.

prove(_,_,Depth) :-
  Depth < 0,!,fail.
prove(true,_,_).
prove((A,B),Anc,Depth) :-
   prove(A,Anc,Depth),
   prove(B,Anc,Depth).
prove((A;B),Anc,Depth) :- !,
   prove(A,Anc,Depth);
   prove(B,Anc,Depth).
prove(H,Anc,_) :-
   compl_member(H,Anc).
prove(H,Anc,Depth) :-
   functor(H,Name,Arity),
   functor(NewH,Name,Arity),
   clause(NewH,B),
   unify_with_occurs_check(NewH,H),
   Depth1 is Depth - 1,
   prove(B,[H|Anc],Depth1).

/**************************************************************/

gen_depth(1).
gen_depth(Depth) :-
   gen_depth(Depth1),
   Depth is Depth1 + 1.

compl_member(E,L) :-
   negate(E,NegE),
   member_occurs_check(NegE,L).

negate(L,NegL) :-
  L =.. [P|Args],
  name(not_,Not),
  name(P,PLst),
  (append(Not,PredLst,PLst) ->
      name(Pred,PredLst);
   append(Not,PLst,PLst1),
   name(Pred,PLst1)
  ),
  NegL =.. [Pred|Args].

member_occurs_check(E,[E1|_]) :- unify_with_occurs_check(E,E1).
member_occurs_check(E,[_|T]) :- member_occurs_check(E,T).
