/******************************************************************/
%
%   This is an SWI-Prolog version of the program 21.2 (game Nim)
%   from the book: Sterling L., Shapiro E., "The Art of Prolog",
%   The MIT Press, 1999.
%
/******************************************************************/

play :-
        initialize(State,Player),
        display_state(State),
        play(State,Player).
play(State,Player) :-
        final_state(State,Player,Result), !,
        announce(Result).
play(State,Player) :-
        choose_move(State,Player,Move),
        display_move(Player,Move),
        move(Move,State,State1),
        display_state(State1),
        next_player(Player,Player1), !,
        play(State1,Player1).
choose_move(State,computer,Move) :-
        evaluate(State,Safety,Sum),
        decide_move(Safety,State,Sum,Move).
choose_move(State,human,Move) :- ask(State,Move).

/*********************************************************/

initialize([1,3,5,7],computer).

display_state(State) :-
   write('Game state: '),
   write(State),
   write('.'), nl, nl.
   
display_move(human,_) :- !.
display_move(computer,Move) :-
   write('My move is: '),
   write(Move),
   write('.'), nl.

final_state([],Player,Player).

announce(computer) :- write('You won! Congratulations!'), nl.
announce(human) :- write('I won...'), nl.

ask(State,Move) :-
   repeat,
   write('Your move > '),
   read(Move),
   legal(Move,State).
   
legal((K,N),State) :-
   nth1(K,State,M),
   N =< M.
legal(Move,State) :-
  write('The move: '),
  write(Move),
  write(' is illegal in the state '),
  write(State), write('.'), nl,
  fail.
  
evaluate(State,Safety,Sum) :-
   nim_sum(State,[],Sum),
   safety(Sum,Safety).
   
safety(Sum,safe) :- zero(Sum), !.
safety(Sum,unsafe) :- not(zero(Sum)),!.

decide_move(safe,_,_,(1,1)).
decide_move(unsafe,State,Sum,Move) :-
   safe_move(State,Sum,Move).
   
move((K,M),[N|Ns],[N|Ns1]) :-
   K > 1,
   K1 is K-1,
   move((K1,M),Ns,Ns1).
move((1,N),[N|Ns],Ns).
move((1,M),[N|Ns],[N1|Ns]) :-
   N > M,
   N1 is N - M.

next_player(computer,human).
next_player(human,computer).

nim_sum([N|Ns],Bs,Sum) :-
   binary(N,Ds),
   nim_add(Ds,Bs,Bs1),
   nim_sum(Ns,Bs1,Sum).
nim_sum([],Sum,Sum).
   
nim_add(Bs,[],Bs).
nim_add([],Bs,Bs).
nim_add([B|Bs],[C|Cs],[D|Ds]) :-
   D is (B+C) mod 2,
   nim_add(Bs,Cs,Ds).
   
binary(1,[1]).
binary(N,[D|Ds]) :-
   N > 1,
   D is N mod 2,
   N1 is N // 2,
   binary(N1,Ds).
   
decimal(Ds,N) :- decimal(Ds,0,1,N).

decimal([],N,_,N).
decimal([D|Ds],A,T,N) :-
   A1 is A+D*T,
   T1 is T*2,
   decimal(Ds,A1,T1,N).
   
zero([]).
zero([0|Zs]) :- zero(Zs).

safe_move(Piles,NimSum,Move) :-
   safe_move(Piles,NimSum,1,Move).

safe_move([Pile|_],NimSum,K,(K,M)) :-
   binary(Pile,Bs),
   can_zero(Bs,NimSum,Ds,0),
   decimal(Ds,M).
safe_move([_|Piles],NimSum,K,Move) :-
   K1 is K+1,
   safe_move(Piles,NimSum,K1,Move).
   
can_zero([],NimSum,[],0) :- zero(NimSum).
can_zero([_|Bs],[0|NimSum],[C|Ds],C) :- can_zero(Bs,NimSum,Ds,C).
can_zero([B|Bs],[1|NimSum],[D|Ds],C) :-
   D is 1-B*C,
   C1 is 1-B,
   can_zero(Bs,NimSum,Ds,C1).
